from whacc import utils, image_tools, transfer_learning, analysis
import matplotlib.pyplot as plt
from tensorflow.keras.layers import Dense, Dropout
from tensorflow.keras.models import Model
from tensorflow import keras
from sklearn.utils import class_weight
import time
from pathlib import Path
import os
import copy
import numpy as np
from tensorflow.keras import applications

from pathlib import Path
import shutil
import zipfile
from datetime import datetime
import pytz
import json
from whacc import model_maker

from whacc.model_maker import *
import itertools

import tensorflow as tf
from tensorflow import keras
import matplotlib.pyplot as plt
import numpy as np
import h5py
from whacc import image_tools
from whacc import utils
import copy
import time
import os
import pdb
import glob
from tqdm.contrib import tzip
import scipy.io as spio
import h5py
from tqdm.notebook import tqdm
from matplotlib import cm

import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
import numpy as np
import matplotlib.patches as mpatches
from scipy.signal import medfilt
import cv2
from tqdm import tqdm
from pathlib import Path


def find_trials_with_suspicious_predictions(frame_nums, pred_bool):
    all_lens = []
    for i, (k1, k2) in enumerate(utils.loop_segments(frame_nums)):
        vals = pred_bool[k1:k2]
        a, b = utils.group_consecutives(vals, step=0)
        y, x = np.histogram([len(k) for k in a], np.linspace(1, 5, 5))
        all_lens.append(y)
    all_lens = np.asarray(all_lens)
    tmp_weights = [3, 3, 2, 1]
    all_lens = all_lens * np.asarray(tmp_weights)
    sorted_worst_estimated_trials = np.flip(np.argsort(np.nanmean(all_lens, axis=1)))
    return sorted_worst_estimated_trials


def foo_rename(instr):
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]


def foo_open_folder(indir):
    indir = foo_rename(indir)
    if os.path.isfile(indir):
        indir = os.path.dirname(indir)
    utils.open_folder(indir)


def split_h5_based_on_trial_inds(h5_file, new_name, trial_inds_to_keep, in_range=None, color_channel=True,
                                 overwrite_if_file_exists=False, labels_key='labels'):
    frame_nums = image_tools.get_h5_key_and_dont_concatenate([h5_file], 'frame_nums')[0]
    h5_creator = image_tools.h5_iterative_creator(new_name,
                                                  overwrite_if_file_exists=overwrite_if_file_exists,
                                                  close_and_open_on_each_iteration=True,
                                                  color_channel=color_channel)
    new_frame_nums = []
    all_frames = np.asarray([])
    with h5py.File(h5_file, 'r') as h:
        LS = utils.loop_segments(frame_nums, returnaslist=True)
        for i in tqdm(trial_inds_to_keep):
            k1 = LS[0][i]
            k2 = LS[1][i]
            if in_range is not None:
                pole_up = in_range[k1:k2]
                pole_up = np.where(pole_up == 1)[0]
                k2 = k1 + pole_up[-1] + 1
                k1 = k1 + pole_up[0]

            all_frames = np.concatenate((all_frames, np.asarray(range(k1, k2))))
            h5_creator.add_to_h5(h['images'][k1:k2], h[labels_key][k1:k2])
            new_frame_nums.append(k2 - k1)

    all_frames = np.asarray(all_frames).flatten()
    frame_nums = np.asarray(new_frame_nums).flatten()

    with h5py.File(h5_creator.h5_full_file_name, 'r+') as h2:
        if 'all_frames' not in h2.keys():
            h2.create_dataset('og_frame_inds', data=all_frames)
        else:
            del h['all_frames']
            h2.create_dataset('og_frame_inds', data=all_frames)
        if 'frame_nums' not in h2.keys():
            h2.create_dataset('frame_nums', data=frame_nums)
        else:
            del h['frame_nums']
            h2.create_dataset('frame_nums', data=frame_nums)
    return h5_creator.h5_full_file_name

# basic variables
model_ind = 45
threshold = .5

h5_file = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED/AH1131_200326___ALT_LABELS.h5'
# H5 file with the model predictions in it
## tmp1 = utils.lister_it(utils.print_h5_keys(h5_file, 1, 0), keep_strings='MODEL_3_regular', remove_string='viterbi')
## key_name = tmp1[model_ind]


key_name = 'MODEL_3_regular 80 border aug 0 to 9__ResNet50V2__3lag__regular__acc_test max__epoch 6__L_ind3__LABELS_-_2021_07_22_06_01_29'
h5_file_IMG = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/curation_for_auto_curator/DATA_FULL/data_AH1131_200326__/3lag/AH1131_200326___3lag.h5'
# H5 file with the images in it

in_range = image_tools.get_h5_key_and_concatenate(h5_file, 'in_range')
# to crop out pole down times

# get the basic variables needed to pick trials
pred_bool = image_tools.get_h5_key_and_concatenate(h5_file, key_name)
pred_bool = utils.confidence_score_to_class(pred_bool, thresh_in = 0.5)
pred_bool[np.invert(in_range.astype(bool))] = -1

trial_nums_and_frame_nums = image_tools.get_h5_key_and_concatenate(h5_file, 'trial_nums_and_frame_nums')
frame_nums = trial_nums_and_frame_nums[1, :].astype(int)
# frame nums to cute the frames based on trials

sorted_worst_estimated_trials = find_trials_with_suspicious_predictions(frame_nums, pred_bool)
num_trials_ = 10

bd = h5_file_IMG.split('.h5')[0] + os.path.sep
new_name_ = bd+'new_h5_to_curate_by_hand.h5'
h5 = split_h5_based_on_trial_inds(h5_file_IMG, new_name_, sorted_worst_estimated_trials[:num_trials_],
                                             in_range=in_range,
                                             color_channel=True, overwrite_if_file_exists=True)

# split the sata into training and test
Path(bd).mkdir(parents=True, exist_ok=True)
split_h5s = image_tools.split_h5_loop_segments([h5],
                                               split_percentages=[.5, .5],  # **********$%$%$%$%$%%$
                                               temp_base_name=[bd + 'train',
                                                               bd + 'val'],
                                               chunk_size=10000,
                                               add_numbers_to_name=False,
                                               disable_TQDM=False,
                                               set_seed=0,
                                               color_channel=True,
                                               force_random_each_frame=False)# force_random_each_frame makes it super slow only needed for rare cases



# from whacc import subset_H5_generator
from whacc import utils, image_tools
from whacc.subset_h5_generator import subset_h5_generator
import numpy as np
import matplotlib.pyplot as plt
import cv2
H5_list = utils.get_h5s('/Users/phil/Dropbox/Autocurator/testing_data/MP4s/')

H5_list = ['/Volumes/GoogleDrive/My Drive/Colab data/curation_for_auto_curator/H5_data/ANM234232_140120_AH1030_AAAA_a.h5']
# tmp1 = '/Users/phil/Downloads/AH1120_200322__.h5'

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

to_pred_h5s = '/Volumes/GoogleDrive/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED'
H5_list_subset = utils.get_h5s(to_pred_h5s)
h5_file = H5_list_subset[-1]
tmp1 = utils.lister_it(utils.print_h5_keys(h5_file, 1, 0), keep_strings='MODEL_3_regular', remove_string='viterbi')
key_name = tmp1[45]
label_key_or_array = image_tools.get_h5_key_and_concatenate(h5_file, key_name).flatten()
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
# label_key_or_array = 'labels'
sg = subset_h5_generator(H5_list[0], label_key_or_array)

sg.get_example_segments(seg_len_before_touch=8,
                        seg_len_after_touch=8,
                        min_y=.2,
                        max_y=.5,
                        num_to_sample=10,
                        min_seg_size=4,
                        start_and_stop_pole_times=[1300, 3000])

# you want a lot of brown which means the model isnt sure and these are teh best frames to correct
sg.plot_all_onset_or_offset(sg.onset_list, fig_size = [10, 10])
sg.plot_all_onset_or_offset(sg.offset_list, fig_size = [10, 10])

import matplotlib.pyplot as plt
import numpy as np
plt.figure()
plt.hist(sg.labels, 100) # histogram pf labels. should be mostly 0's and 1's if not then the predictions might be really bad

plt.figure()
for i, k in enumerate(sg.onset_list):
    plt.plot(np.asarray(range(len(k)))+len(k)*i, k, 'ok')
for i, k in enumerate(sg.offset_list):
    plt.plot(np.asarray(range(len(k)))+len(k)*i, k, 'or')

len_c = []
for k in sg.chunks:
    if len(k)<=100:
        len_c.append(len(k))
plt.figure()
plt.hist(len_c, 100)

save_on = False
if save_on:
    sg.save_subset_h5_file()


# change the labels into the correct human curated labels
H5_list = ['/Volumes/GoogleDrive/My Drive/Colab data/curation_for_auto_curator/H5_data/ANM234232_140120_AH1030_AAAA_a.h5']
to_pred_h5s = '/Volumes/GoogleDrive/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED'
H5_list_subset = utils.get_h5s(to_pred_h5s)
h5_file = H5_list_subset[-1]
real_bool = image_tools.get_h5_key_and_concatenate(h5_file, '[0, 1]- (no touch, touch)')
# '/Volumes/GoogleDrive/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED/ANM234232_140120_AH1030_AAAA_a_ALT_LABELS.h5'

vals = image_tools.get_h5_key_and_concatenate(sg.file_save_name, 'all_inds')
vals = real_bool[vals]
utils.add_to_h5(sg.file_save_name, 'labels', vals, overwrite_if_exists=True)


utils.get_class_info(sg)



from whacc.image_tools import *
import copy
import pdb


class ImageBatchGenerator_feature_array(keras.utils.Sequence):

    def __init__(self, time_length, batch_size, h5_file_list, label_key='labels', feature_len=None,
                 label_index_to_lstm_len=None, edge_value=-1, remove_any_time_points_with_edges = True):
        """
        
        Parameters
        ----------
        time_length : total time points
        batch_size : batch output for generator 
        h5_file_list : list of h5 strings or single h5 string 
        label_key : where y output comes from 
        feature_len : length of the features per time point
        label_index_to_lstm_len : determines look back and look forward index refers to where the 'current' time point is
        within the range of look_back_len; e.g. look_back_len = 7 label_index_to_lstm_len = 3 (middle index of 7) then 
        time point 0 will be at 3 and index 0, 1, 2 will be the past values and index 4, 5, 6 will be the future values.
        look_back_len = 7 label_index_to_lstm_len = 0 (first index) then current time point will be at index 0 and all
        other time point (1, 2, 3, 4, 5, 6) will be future values. Default is middle time point
        edge_value : what to replace the edge values with, when time shifting you will have edges with no value, this
        will replace those values with this number.
        remove_any_time_points_with_edges : if true then batch size will not be the actual batch size it will be batch
        size - the number of time points with edges in them, x and y will still match and this method is preferred for
        training due to it not including unknown values.
        """
        assert time_length % 2 == 1, "number of images must be odd"
        if label_index_to_lstm_len is None:
            label_index_to_lstm_len = time_length // 2  # in the middle
        h5_file_list = utils.make_list(h5_file_list, suppress_warning=True)
        num_frames_in_all_H5_files = get_total_frame_count(h5_file_list)
        file_inds_for_H5_extraction = batch_size_file_ind_selector(
            num_frames_in_all_H5_files, batch_size)
        subtract_for_index = reset_to_first_frame_for_each_file_ind(
            file_inds_for_H5_extraction)
        self.remove_any_time_points_with_edges = remove_any_time_points_with_edges
        self.label_key = label_key
        self.batch_size = batch_size
        self.H5_file_list = h5_file_list
        self.num_frames_in_all_H5_files = num_frames_in_all_H5_files
        self.file_inds_for_H5_extraction = file_inds_for_H5_extraction
        self.subtract_for_index = subtract_for_index
        self.label_index_to_lstm_len = label_index_to_lstm_len
        self.lstm_len = time_length
        self.feature_len = feature_len
        self.edge_value = edge_value
        if remove_any_time_points_with_edges:
            self.edge_value = np.nan
            print('remove_any_time_points_with_edges == True : forcing edge_value to np.nan to aid in removing these time points')


        self.get_frame_edges()
        # self.full_edges_mask = self.full_edges_mask - (self.lstm_len // 2 - self.label_index_to_lstm_len)

    def __getitem__(self, num_2_extract):
        h = self.H5_file_list
        i = self.file_inds_for_H5_extraction
        all_edges = self.all_edges_list[np.int(i[num_2_extract])]
        H5_file = h[np.int(i[num_2_extract])]
        num_2_extract_mod = num_2_extract - self.subtract_for_index[num_2_extract]

        with h5py.File(H5_file, 'r') as h:
            b = self.lstm_len // 2
            tot_len = h['images'].shape[0]

            # assert tot_len - b > self.batch_size, "reduce batch size to be less than total length of images minus floor(lstm_len) - 1, MAX->" + str(
            #     tot_len - b - 1)

            i1 = num_2_extract_mod * self.batch_size - b
            i2 = num_2_extract_mod * self.batch_size + self.batch_size + b
            edge_left_trigger = abs(min(i1, 0))
            edge_right_trigger = min(abs(min(tot_len - i2, 0)), b)
            x = h['images'][max(i1, 0):min(i2, tot_len)]
            if edge_left_trigger + edge_right_trigger > 0:  # in case of edge cases
                pad_shape = list(x.shape)
                pad_shape[0] = edge_left_trigger + edge_right_trigger
                pad = np.zeros(pad_shape).astype('float32')
                if edge_left_trigger > edge_right_trigger:
                    x = np.concatenate((pad, x), axis=0)
                else:
                    x = np.concatenate((x, pad), axis=0)

            s = list(x.shape)
            s.insert(1, self.lstm_len)
            out = np.zeros(s).astype('float32')  # before was uint8
            Z = self.label_index_to_lstm_len - self.lstm_len // 2
            for i in range(self.lstm_len):
                i_temp = i
                i = i - Z
                i1 = max(0, b - i)
                i2 = min(s[0], s[0] + b - i)
                i3 = max(0, i - b)
                i4 = min(s[0], s[0] + i - b)
                # print('take ', i3, ' to ', i4, ' and place in ', i1, ' to ', i2)
                out[i1:i2, i_temp, ...] = x[i3:i4, ...]

            out = out[b:s[0] - b, ...]
            i1 = num_2_extract_mod * self.batch_size
            i2 = num_2_extract_mod * self.batch_size + self.batch_size
            raw_Y = h[self.label_key][i1:i2]

            adjust_these_edge_frames = np.intersect1d(all_edges.flatten(), np.arange(i1, i2))
            b2 = b - self.label_index_to_lstm_len  # used to adjust mask postion based on where the center value is
            for atef in adjust_these_edge_frames:
                mask_ind = np.where(atef == all_edges)[1][0]
                mask_ind = mask_ind - b2
                mask_ind = mask_ind % self.full_edges_mask.shape[0]  # wrap around index

                mask_ = self.full_edges_mask[mask_ind]
                mask_ = mask_ == 1
                out_ind = atef + i1 - b2
                out_ind = out_ind % out.shape[0]  # wrap around index
                out[out_ind][mask_] = self.edge_value


            s = out.shape
            out = np.reshape(out, (s[0], s[1] * s[2]))
            if self.remove_any_time_points_with_edges:
                keep_inds = ~np.isnan(np.mean(out, axis = 1))
                out = out[keep_inds]
                raw_Y = raw_Y[keep_inds]

            return out, raw_Y

    def __len__(self):
        return len(self.file_inds_for_H5_extraction)

    def getXandY(self, num_2_extract):
        """

        Parameters
        ----------
        num_2_extract :


        Returns
        -------

        """
        rgb_tensor, raw_Y = self.__getitem__(num_2_extract)
        return rgb_tensor, raw_Y

    def image_transform(self, raw_X):
        """input num_of_images x H x W, image input must be grayscale
        MobileNetV2 requires certain image dimensions
        We use N x 61 x 61 formated images
        self.IMG_SIZE is a single number to change the images into, images must be square

        Parameters
        ----------
        raw_X :


        Returns
        -------


        """
        # kept this cause this is the format of the image generators I know this is redundant
        rgb_batch = copy.deepcopy(raw_X)
        rgb_tensor = rgb_batch
        self.IMG_SHAPE = (self.feature_len)
        return rgb_tensor

    def get_frame_edges(self):
        self.all_edges_list = []
        b = self.lstm_len // 2

        s = [b * 2, self.lstm_len, self.feature_len]
        for H5_file in self.H5_file_list:
            with h5py.File(H5_file, 'r') as h:
                full_edges_mask = np.ones(s)
                tmp1 = np.arange(1, self.lstm_len)
                front_edge = tmp1[:self.label_index_to_lstm_len]
                back_edge = tmp1[:self.lstm_len - self.label_index_to_lstm_len - 1]

                edge_ind = np.flip(front_edge)
                for i in front_edge:
                    # print(i - 1, ':', edge_ind[i - 1])
                    # print(full_edges_mask[i - 1, :edge_ind[i - 1], ...].shape)
                    # print('\n')
                    full_edges_mask[i - 1, :edge_ind[i - 1], ...] = np.zeros_like(
                        full_edges_mask[i - 1, :edge_ind[i - 1], ...])

                edge_ind = np.flip(back_edge)
                for i in back_edge:
                    # print(-i, -edge_ind[i - 1], ':')
                    # print(full_edges_mask[-i, -edge_ind[i - 1]:, ...].shape)
                    # print('\n')
                    full_edges_mask[-i, -edge_ind[i - 1]:, ...] = np.zeros_like(
                        full_edges_mask[-i, -edge_ind[i - 1]:, ...])

                all_edges = []
                for i1, i2 in utils.loop_segments(h['frame_nums']):  # 0, 1, 3998, 3999 ; 4000, 4001, 7998, 7999; ...
                    edges = (np.asarray([[i1], [i2 - b]]) + np.arange(0, b).T).flatten()
                    all_edges.append(edges)

                all_edges = np.asarray(all_edges)
            self.all_edges_list.append(all_edges)
            # pdb.set_trace()
            full_edges_mask = full_edges_mask.astype(int)
            self.full_edges_mask = full_edges_mask == 0


h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing_features_data/feature_data/regular_80_border/DATA_FULL/3lag/holy_test_set_10_percent_3lag.h5'
# h5 = '/Users/phil/Desktop/temp.h5'
a = ImageBatchGenerator_feature_array(7, 4000, h5, label_key='labels', feature_len=2048,
                                      label_index_to_lstm_len=3, edge_value=-1, remove_any_time_points_with_edges = False)
x, y = a.__getitem__(0)
# # utils.np_stats(x)
# # s = x.shape;
# # x2 = np.reshape(x, (s[0], s[1] * s[2]))


plt.figure()
plt.plot(y[2200:2300])

plt.figure()
plt.imshow(x[2200:2300, ::512].T)





import cv2
import matplotlib.pyplot as plt
import numpy as np
from PIL import Image

cap = cv2.VideoCapture("/Users/phil/Dropbox/Autocurator/testing_data/MP4s/AH1131X26032020ses338_SAMSON/AH1131X26032020ses338-52.mp4")
img_to_load = "/Users/phil/Dropbox/Autocurator/testing_data/MP4s/AH1131X26032020ses338_SAMSON/template_img.png"
template_image = np.asarray(Image.open(img_to_load))
amount_of_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)

frame_number = 2000

cap.set(cv2.CAP_PROP_POS_FRAMES, frame_number-1)
res, frame = cap.read()

gpu_frame = cv2.cuda_GpuMat()

res = cv2.matchTemplate(frame, template_image, 'cv2.TM_CCOEFF')












#
# plt.figure()
#
# plt.imshow(frame)
#
#
#
# frame_number = 1
#
# cap.set(cv2.CAP_PROP_POS_FRAMES, frame_number-1)
# res, frame = cap.read()
# plt.figure()
# plt.imshow(frame)
#
#
#
# cap.set(cv2.CAP_PROP_POS_FRAMES, -9999)
#
#
#
#
# start_frame = 2000
#
# tmp1 = np.arange(start_frame, amount_of_frames)
# tmp2 = np.flip(np.arange(0, start_frame))
#
# tmp1



from whacc import utils, image_tools

import copy
import numpy as np
import pandas as pd
import h5py

if utils.isnotebook():
    from tqdm.notebook import tqdm
else:
    from tqdm import tqdm


class feature_maker():
    def __init__(self, h5_in, frame_num_ind=None, frame_nums=None, operational_key='FD__original', disable_tqdm=False,
                 delete_if_exists=False):
        """

        Parameters
        ----------
        h5_in : h5 string pointing to the h4 file with the data to transform
        frame_num_ind : int referencing the frame num  ind to transform, note: if save_it is one ALL data is converted
        automatically and saved in h5. frame_num_ind only works when you call with
        frame_nums : default None, auto looks for key 'frame_nums' in h5 file or you can insert your own
        operational_key : the data array key to be transformed
        disable_tqdm : default False, when save_it is True it will show a loading bar with the progress unless set to true
        delete_if_exists : default False, when calling a function with save_it as True, you can choose to overwrite that
        data by setting this value to True

        Returns
        -------
        feature maker class

        Examples
        ________

        h5_in = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'

        FM = feature_maker(h5_in, frame_num_ind=2, delete_if_exists = True) #set frame_num_ind to an int so that we can quickly look at the transformation
        data, data_name = FM.shift(5, save_it=False) # shift it 5 to the right, fills in with nan as needed, look at just the 5th frame num set (5th video)
        # to see how it looks
        print('the below name will be used to save the data in the the h5 when calling functions with save_it=True')
        print(data_name)
        FM.shift(5, save_it=True) # now lets save it

        data, data_name_rolling_mean_100 = FM.rolling(100, 'mean', save_it=True)  #lets do the rolling mean and save it

        data, data_name = FM.operate('diff', kwargs={'periods': 1}, save_it=False) # lets perform a diff operation
        print(data_name)
        print(data)
        FM.operate('diff', kwargs={'periods': -1}, save_it=True) # now lets save it and save it

        # now lets change the operational key so we can transform some data we just transformed, specifically the rolling mean 10
        FM.set_operation_key(data_name_rolling_mean_100)

        data, data_name_diff_100_mean = FM.operate('diff', kwargs={'periods': -50}, save_it=False) # lets check how the name will change
        print(data_name_diff_100_mean)
        print("notice the FD__ twice, this means the data has been transformed twice")
        print('also notice that how the data was transformed can be seen because it is split up by ____ (4 underscores)')

        data, data_name_diff_100_mean = FM.operate('diff', kwargs={'periods': -50}, save_it=True) # save it

        a = utils.print_h5_keys(h5_in, 1, 1)
        key_name_list = ['FD__original', 'FD__FD__FD__original_rolling_mean_W_100_SFC_0_MP_100_____diff_periods_-50____', 'FD__FD__original_rolling_mean_W_100_SFC_0_MP_100____']
        with h5py.File(h5_in, 'r+') as h:
            for k in key_name_list:
                plt.plot(h[k][:8000, 0])
        """
        if utils.h5_key_exists(h5_in, 'has_data_been_randomized'):
            assert image_tools.get_h5_key_and_concatenate(h5_in, 'has_data_been_randomized') is False, """this data has been randomized, it is not fit to perform temporal operations on, search for key 'has_data_been_randomized' for more info"""

        self._frame_num_ind_save_ = None
        self.disable_tqdm = disable_tqdm
        self.h5_in = h5_in
        self.frame_num_ind = frame_num_ind
        self.delete_if_exists = delete_if_exists
        self.operational_key = operational_key
        if frame_nums is None:
            frame_nums = image_tools.get_h5_key_and_concatenate(h5_in, 'frame_nums')
        self.all_frame_nums = frame_nums
        self.len_frame_nums = len(self.all_frame_nums)
        self.set_data_and_frame(frame_num_ind)

    def set_data_inds(self, ind):
        tmp_inds = np.asarray(utils.loop_segments(self.all_frame_nums, returnaslist=True))
        if ind is None:
            self.data_inds = [tmp_inds[0][0], tmp_inds[-1][-1]]
        else:
            self.data_inds = tmp_inds[:, ind]

    def set_operation_key(self, key_name=None):
        if key_name is not None:
            self.operational_key = key_name
        with h5py.File(self.h5_in, 'r') as h:
            self.full_data_shape = np.asarray(h[self.operational_key].shape)
            if self.frame_num_ind is None:
                self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][:]))
            else:
                a = self.data_inds
                self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][a[0]:a[1]]))

    def init_h5_data_key(self, data_key, delete_if_exists=False):
        key_exists = utils.h5_key_exists(self.h5_in, data_key)
        assert not (
                key_exists and not delete_if_exists), "key exists, if you want to overwrite set 'delete_if_exists' = True"
        with h5py.File(self.h5_in, 'r+') as x:
            if key_exists and delete_if_exists:
                print('deleting key to overwrite it')
                del x[data_key]
            x.create_dataset_like(data_key, x[self.operational_key])

    def rolling(self, window, operation, shift_from_center=0, min_periods=None, save_it=False, kwargs={}):
        """

        Parameters
        ----------
        window : window size int
        operation : a string with the operation e.g. 'mean' or 'std' see pandas docs for rolling
        shift_from_center : default 0 but can shift as needed
        min_periods : default is equal to win length, only allows operation when we have this many data points. so deals with
        the edges
        save_it : bool, False, loop through frame nums and save to h5 file
        kwargs : dict of args that can be applied to 'operation' function

        Returns
        -------

        """
        if min_periods is None:
            min_periods = window
        add_name_list = ['FD__' + self.operational_key + '_rolling_',
                         '_W_' + str(window) + '_SFC_' + str(shift_from_center) + '_MP_' + str(min_periods) + '____']
        add_name_str = operation.join(add_name_list)
        if save_it:
            self._save_it_(self.rolling, window, operation, shift_from_center, min_periods, False, kwargs)
            return None, add_name_str
        data_frame = self.data.copy()
        for i1, i2 in utils.loop_segments(self.frame_nums):
            df_rolling = self.data[i1:i2].rolling(window=window, min_periods=min_periods, center=True)
            tmp_func = eval('df_rolling.' + operation)
            data_frame[i1:i2] = tmp_func(**kwargs).shift(shift_from_center).astype(np.float32)
        return data_frame, add_name_str

    def shift(self, shift_by, save_it=False):
        """

        Parameters
        ----------
        shift_by : amount to shift by
        save_it :  bool, False, loop through frame nums and save to h5 file

        Returns
        -------

        """
        add_name_str = 'FD__' + self.operational_key + '_shift_' + str(shift_by) + '____'
        if save_it:
            self._save_it_(self.shift, shift_by)
            return None, add_name_str

        data_frame = self.data.copy()
        for i1, i2 in utils.loop_segments(self.frame_nums):
            data_frame[i1:i2] = self.data[i1:i2].shift(shift_by).astype(np.float32)
        return data_frame, add_name_str

    def operate(self, operation, save_it=False, extra_name_str='', kwargs={}):
        """

        Parameters
        ----------
        operation : a string with the operation e.g. 'mean' or 'std' or 'diff' see pandas operation on  dataframes
        save_it :  bool, False, loop through frame nums and save to h5 file
        extra_name_str : add to key name string if desired
        kwargs : dict of args that can be applied to 'operation' function see pandas operation on  dataframes

        Returns
        -------

        """
        add_name_str_tmp = self.dict_to_string_name(kwargs)
        add_name_str = 'FD__' + self.operational_key + '_' + operation + '_' + add_name_str_tmp + extra_name_str + '____'
        if save_it:
            self._save_it_(self.operate, operation, False, extra_name_str, kwargs)
            return None, add_name_str
        data_frame = self.data.copy()
        for i1, i2 in utils.loop_segments(self.frame_nums):
            data_frame[i1:i2] = eval('self.data[i1:i2].' + operation + '(**kwargs).astype(np.float32)')
        return data_frame, add_name_str

    @staticmethod
    def dict_to_string_name(in_dict):
        """
        used to transform dict into a string name for naming h5 keys
        Parameters
        ----------
        in_dict : dict

        Returns
        -------
        string
        """
        str_list = []
        for k in in_dict:
            str_list.append(k)
            str_list.append(str(in_dict[k]))
        return '_'.join(str_list)

    def set_data_and_frame(self, ind):
        self.frame_num_ind = ind
        if ind is None:
            self.frame_nums = copy.deepcopy(self.all_frame_nums)
        else:
            self.frame_nums = [self.all_frame_nums[ind]]
        self.set_data_inds(ind)
        self.set_operation_key()

    # tqdm(sorted(random_frame_inds[i]), disable=disable_TQDM, total=total_frames, initial=cnt1)
    def _save_it_(self, temp_funct, *args):
        self._frame_num_ind_save_ = copy.deepcopy(self.frame_num_ind)
        with h5py.File(self.h5_in, 'r+') as h:
            for k in tqdm(range(self.len_frame_nums), disable=self.disable_tqdm):
                self.set_data_and_frame(k)
                data, key_name = temp_funct(*args)
                if k == 0:
                    self.init_h5_data_key(key_name, delete_if_exists=self.delete_if_exists)
                    print('making key, ' + key_name)
                a = self.data_inds
                h[key_name][a[0]:a[1]] = data
        self.set_data_and_frame(self._frame_num_ind_save_)  # set data back to what it was when user set it

#
# class feature_maker():
#     def __init__(self, h5_in, frame_num_ind=None, frame_nums=None, operational_key='FD__original', disable_tqdm=False, delete_if_exists=False):
#         """
#
#         Parameters
#         ----------
#         h5_in : h5 string pointing to the h4 file with the data to transform
#         frame_num_ind : int referencing the frame num  ind to transform, note: if save_it is one ALL data is converted
#         automatically and saved in h5. frame_num_ind only works when you call with
#         frame_nums : default None, auto looks for key 'frame_nums' in h5 file or you can insert your own
#         operational_key : the data array key to be transformed
#         disable_tqdm : default False, when save_it is True it will show a loading bar with the progress unless set to true
#         delete_if_exists : default False, when calling a function with save_it as True, you can choose to overwrite that
#         data by setting this value to True
#
#         Returns
#         -------
#         feature maker class
#
#         Examples
#         ________
#
#         h5_in = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'
#
#         FM = feature_maker(h5_in, frame_num_ind=2, delete_if_exists = True) #set frame_num_ind to an int so that we can quickly look at the transformation
#         data, data_name = FM.shift(5, save_it=False) # shift it 5 to the right, fills in with nan as needed, look at just the 5th frame num set (5th video)
#         # to see how it looks
#         print('the below name will be used to save the data in the the h5 when calling functions with save_it=True')
#         print(data_name)
#         FM.shift(5, save_it=True) # now lets save it
#
#         data, data_name_rolling_mean_100 = FM.rolling(100, 'mean', save_it=True)  #lets do the rolling mean and save it
#
#         data, data_name = FM.operate('diff', kwargs={'periods': 1}, save_it=False) # lets perform a diff operation
#         print(data_name)
#         print(data)
#         FM.operate('diff', kwargs={'periods': -1}, save_it=True) # now lets save it and save it
#
#         # now lets change the operational key so we can transform some data we just transformed, specifically the rolling mean 10
#         FM.set_operation_key(data_name_rolling_mean_100)
#
#         data, data_name_diff_100_mean = FM.operate('diff', kwargs={'periods': -50}, save_it=False) # lets check how the name will change
#         print(data_name_diff_100_mean)
#         print("notice the FD__ twice, this means the data has been transformed twice")
#         print('also notice that how the data was transformed can be seen because it is split up by ____ (4 underscores)')
#
#         data, data_name_diff_100_mean = FM.operate('diff', kwargs={'periods': -50}, save_it=True) # save it
#
#         a = utils.print_h5_keys(h5_in, 1, 1)
#         key_name_list = ['FD__original', 'FD__FD__FD__original_rolling_mean_W_100_SFC_0_MP_100_____diff_periods_-50____', 'FD__FD__original_rolling_mean_W_100_SFC_0_MP_100____']
#         with h5py.File(h5_in, 'r+') as h:
#             for k in key_name_list:
#                 plt.plot(h[k][:8000, 0])
#         """
#         self.disable_tqdm = disable_tqdm
#         self.h5_in = h5_in
#         self.frame_num_ind = frame_num_ind
#         self.delete_if_exists = delete_if_exists
#         self.operational_key = operational_key
#         if frame_nums is None:
#             frame_nums = image_tools.get_h5_key_and_concatenate(h5_in, 'frame_nums')
#         self.all_frame_nums = frame_nums
#         self.len_frame_nums = len(self.all_frame_nums)
#         self.set_data_and_frame(frame_num_ind)
#
#     def set_data_inds(self, ind):
#         tmp_inds = np.asarray(utils.loop_segments(self.all_frame_nums, returnaslist=True))
#         self.data_inds = tmp_inds[:, ind]
#
#     def set_operation_key(self, key_name=None):
#         if key_name is not None:
#             self.operational_key = key_name
#         with h5py.File(self.h5_in, 'r') as h:
#             self.full_data_shape = np.asarray(h[self.operational_key].shape)
#             if self.frame_num_ind is None:
#                 self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][:]))
#             else:
#                 a = self.data_inds
#                 self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][a[0]:a[1]]))
#
#     def init_h5_data_key(self, data_key, delete_if_exists=False):
#         key_exists = utils.h5_key_exists(self.h5_in, data_key)
#         assert not (
#                 key_exists and not delete_if_exists), "key exists, if you want to overwrite set 'delete_if_exists' = True"
#         with h5py.File(self.h5_in, 'r+') as x:
#             if key_exists and delete_if_exists:
#                 print('deleting key to overwrite it')
#                 del x[data_key]
#             x.create_dataset_like(data_key, x[self.operational_key])
#
#     def rolling(self, window, operation, shift_from_center=0, min_periods=None, save_it=False, kwargs={}):
#         """
#
#         Parameters
#         ----------
#         window : window size int
#         operation : a string with the operation e.g. 'mean' or 'std' see pandas docs for rolling
#         shift_from_center : default 0 but can shift as needed
#         min_periods : default is equal to win length, only allows operation when we have this many data points. so deals with
#         the edges
#         save_it : bool, False, loop through frame nums and save to h5 file
#         kwargs : dict of args that can be applied to 'operation' function
#
#         Returns
#         -------
#
#         """
#         if min_periods is None:
#             min_periods = window
#         add_name_list = ['FD__' + self.operational_key + '_rolling_',
#                          '_W_' + str(window) + '_SFC_' + str(shift_from_center) + '_MP_' + str(min_periods) + '____']
#         add_name_str = operation.join(add_name_list)
#         if save_it:
#             self._save_it_(self.rolling, window, operation, shift_from_center, min_periods, False, kwargs)
#             return None, add_name_str
#         data_frame = self.data.copy()
#         for i1, i2 in utils.loop_segments(self.frame_nums):
#             df_rolling = self.data[i1:i2].rolling(window=window, min_periods=min_periods, center=True)
#             tmp_func = eval('df_rolling.' + operation)
#             data_frame[i1:i2] = tmp_func(**kwargs).shift(shift_from_center).astype(np.float32)
#         return data_frame, add_name_str
#
#     def shift(self, shift_by, save_it=False):
#         """
#
#         Parameters
#         ----------
#         shift_by : amount to shift by
#         save_it :  bool, False, loop through frame nums and save to h5 file
#
#         Returns
#         -------
#
#         """
#         add_name_str = 'FD__' + self.operational_key + '_shift_' + str(shift_by) + '____'
#         if save_it:
#             self._save_it_(self.shift, shift_by)
#             return None, add_name_str
#
#         data_frame = self.data.copy()
#         for i1, i2 in utils.loop_segments(self.frame_nums):
#             data_frame[i1:i2] = self.data[i1:i2].shift(shift_by).astype(np.float32)
#         return data_frame, add_name_str
#
#     def operate(self, operation, save_it=False, extra_name_str='', kwargs={}):
#         """
#
#         Parameters
#         ----------
#         operation : a string with the operation e.g. 'mean' or 'std' or 'diff' see pandas operation on  dataframes
#         save_it :  bool, False, loop through frame nums and save to h5 file
#         extra_name_str : add to key name string if desired
#         kwargs : dict of args that can be applied to 'operation' function see pandas operation on  dataframes
#
#         Returns
#         -------
#
#         """
#         add_name_str_tmp = self.dict_to_string_name(kwargs)
#         add_name_str = 'FD__' + self.operational_key + '_' + operation + '_' + add_name_str_tmp + extra_name_str + '____'
#         if save_it:
#             self._save_it_(self.operate, operation, False,extra_name_str, kwargs)
#             return None, add_name_str
#         data_frame = self.data.copy()
#         for i1, i2 in utils.loop_segments(self.frame_nums):
#             data_frame[i1:i2] = eval('self.data[i1:i2].' + operation + '(**kwargs).astype(np.float32)')
#         return data_frame, add_name_str
#     @staticmethod
#     def dict_to_string_name(in_dict):
#         """
#         used to transform dict into a string name for naming h5 keys
#         Parameters
#         ----------
#         in_dict : dict
#
#         Returns
#         -------
#         string
#         """
#         str_list = []
#         for k in in_dict:
#             str_list.append(k)
#             str_list.append(str(in_dict[k]))
#         return '_'.join(str_list)
#
#     def set_data_and_frame(self, ind):
#         self.frame_nums = [self.all_frame_nums[ind]]
#         self.set_data_inds(ind)
#         self.set_operation_key()
#
#     # tqdm(sorted(random_frame_inds[i]), disable=disable_TQDM, total=total_frames, initial=cnt1)
#     def _save_it_(self, temp_funct, *args):
#         with h5py.File(self.h5_in, 'r+') as h:
#             for k in tqdm(range(self.len_frame_nums), disable=self.disable_tqdm):
#                 self.set_data_and_frame(k)
#                 data, key_name = temp_funct(*args)
#                 if k == 0:
#                     self.init_h5_data_key(key_name, delete_if_exists=self.delete_if_exists)
#                     print('making key, ' + key_name)
#                 a = self.data_inds
#                 h[key_name][a[0]:a[1]] = data
#



# import numpy as np
# # import scipy.io
# import scipy.io as spio
# from google.colab import drive
# import glob
# import matplotlib.pyplot as plt
# import h5py
# import os
# # from functools import partial
# from tqdm import tqdm
# # tqdm = partial(tqdm, position=0, leave=True)
# import difflib
# import copy
# import re
# from matplotlib.colors import ListedColormap
# import itertools
# from pathlib import Path

from whacc import utils

pole_file_list = utils.get_files("/Users/phil/Dropbox/Autocurator/testing_data/MP4s/AH1131X26032020ses338_SAMSON/", '*.mat')


pt = utils.loadmat(pole_file_list[0])



import sys
sys.path.append('/opt/conda/lib/python3.7/site-packages')
bd2 = '/home/jupyter/'

from whacc import utils, image_tools, transfer_learning, analysis
import matplotlib.pyplot as plt
from tensorflow.keras.layers import Dense, Dropout
from tensorflow.keras.models import Model
from tensorflow import keras
import tensorflow as tf
from sklearn.utils import class_weight
import time
from pathlib import Path
import os
import copy
import numpy as np
from tensorflow.keras import applications
from pathlib import Path

import shutil
import zipfile
from datetime import datetime
import pytz
import json


# In[16]:


def build_model(info_dict, labels, model_name_str, base_learning_rate=0.00001, dropout_val=None, class_numbers=None,
                IMG_SIZE=96):
    if class_numbers is None:
        class_numbers = np.unique(labels)
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    base_model.trainable = True
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
                   keras.metrics.AUC(name='auc')]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]

    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
    class_weights = {i: rebalance[i] for i in class_numbers}
    wrap_vars_list = ['class_numbers',
                      'num_classes',
                      'base_learning_rate',
                      'model_name_str',
                      'IMG_SIZE',
                      'dropout_val']
    for k in wrap_vars_list:
        info_dict[k] = eval(k)
    info_dict['class_weights'] = str(class_weights)
    return model, class_weights, info_dict


def label_naming_shorthand_dict(name_key=None):
    label_naming_shorthand_dict = {
        '[0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)': 'on-off_set_and_one_after',
        '[0, 1, 2, 3]- (no touch, touch, onset, offset': 'on-off_set',
        '[0, 1, 2]- (no event, onset, offset)': 'only_on-off_set',
        '[0, 1]- (no touch, touch)': 'regular',
        '[0, 1]- (not offset, offset)': 'only_offset',
        '[0, 1]- (not onset, onset)': 'only_onset'}
    if name_key is None:
        return label_naming_shorthand_dict
    else:
        return label_naming_shorthand_dict[name_key]


def info_dict_wrapper(info_dict, local_dict):
    for k in local_dict.keys():
        info_dict[k] = local_dict[k]
    return info_dict


def make_initial_folder(all_models_directory, unique_h5_train_val_dir):
    single_frame_dir = all_models_directory + os.sep + unique_h5_train_val_dir + os.sep + 'data' + os.sep + 'single_frame'
    Path(single_frame_dir).mkdir(parents=True, exist_ok=True)
    return locals()
    # return single_frame_dir


def get_automated_model_info(BASE_H5, image_source_h5_directory_ending, test_data_dir, data_string_key = "data"):
    tz = pytz.timezone('America/Los_Angeles')
    loc_dt = pytz.utc.localize(datetime.utcnow())
    LA_TIME = loc_dt.astimezone(tz)
    todays_version = LA_TIME.strftime("%Y_%m_%d_%H_%M_%S")
    del tz
    del loc_dt
    del LA_TIME
    a = os.sep
    base_data_dir = BASE_H5 + a + data_string_key + a
    base_dir_all_h5s = BASE_H5 + a + data_string_key + a + 'single_frame' + a
    data_dir = base_data_dir + image_source_h5_directory_ending
    print('\nFOR IMAGES, 0 is train set, 1 is val set')
    print(data_dir)
    image_h5_list = utils.get_h5s(data_dir)
    h5_train = image_h5_list[0]
    h5_val = image_h5_list[1]
    labels_dir = base_data_dir + a + "ALT_LABELS" + a
    print('\nFOR LABELS,0 is train set, 1 is val set')
    label_h5_list = utils.get_h5s(labels_dir)
    print('\nSelect from the following label structures...')
    label_key_name_list = utils.print_h5_keys(label_h5_list[0], return_list=True)
    h5_test_labels = utils.get_h5s(test_data_dir + a + "ALT_LABELS" + a, print_h5_list=False)[0]
    h5_test = utils.get_h5s(test_data_dir + a + image_source_h5_directory_ending + a, print_h5_list=False)[0]
    return locals()


def copy_over_new_labels(label_key_name, image_h5_list, label_h5_list):
    label_key_shorthand = label_naming_shorthand_dict(label_key_name)
    for img_src, lab_src in zip(image_h5_list, label_h5_list):
        utils.copy_h5_key_to_another_h5(lab_src, img_src, label_key_name, 'labels')
    return locals()


# get list of pre trained models to choose from
def get_keras_model_names():
    names_, types_ = utils.get_class_info(applications, return_name_and_type=True)
    model_names = np.asarray(names_)['function' == np.asarray(types_)]
    utils.print_list_with_inds(model_names)
    return model_names


def make_model_save_directory(info_dict, make_folder=True):
    naming_list = ['model_name_str', 'image_source_h5_directory_ending', 'label_key_shorthand', 'todays_version']
    model_save_dir: str = copy.deepcopy(info_dict['BASE_H5'])
    for k in naming_list:
        model_save_dir += os.sep + info_dict[k] + os.sep
    info_dict['model_save_dir'] = model_save_dir
    info_dict['model_save_dir_checkpoints'] = model_save_dir + os.sep + 'checkpoints'
    if make_folder:
        Path(info_dict['model_save_dir_checkpoints']).mkdir(parents=True, exist_ok=True)
    return model_save_dir


def basic_callbacks(save_checkpoint_filepath, monitor='val_loss', patience=10,
                    save_best_only=False, save_weights_only=True, save_freq="epoch", period = 1):
    callbacks = []
    callbacks.append(keras.callbacks.EarlyStopping(monitor=monitor, patience=patience))
    add_path_name = "{loss:.8f}_{epoch:04d}_cp.hdf5"
    add_path_name = "{epoch:04d}_cp.hdf5"
    callbacks.append(keras.callbacks.ModelCheckpoint(
        save_checkpoint_filepath + os.sep + add_path_name,
        monitor=monitor,
        save_best_only=save_best_only,
        save_weights_only=save_weights_only,
        save_freq=save_freq,
        period=period))
    return callbacks
def unzip_and_place_h5s(bd, do_delete_zips = False):
  bd2= ''.join(bd.split('gdrive/My Drive'))
  shutil.copytree(bd, bd2)
  a = utils.get_files(bd2, '*.zip')
  for k in a:
    with zipfile.ZipFile(k, 'r') as zip_ref:
        zip_ref.extractall(os.path.dirname(k))
    if do_delete_zips:
      os.remove(k)
def change_to_local_dir(bd):
  bd2= ''.join(bd.split('gdrive/My Drive'))
  return bd2
def change_to_gdrive_dir(bd):
  bd2= '/content/gdrive/My Drive/'.join(bd.split('/content/'))
  return bd2
def pack_training_info(save_and_plot_history_var):
    training_info = dict()
    for k in ['L', 'all_logs', 'all_colors', 'logs_names', 'markers', 'matching_inds']:
        training_info[k] = eval('save_and_plot_history_var.' + k)
    for k in training_info.keys():
        if 'numpy.ndarray' in str(type(training_info[k])):
            training_info[k] = training_info[k].tolist()
    return training_info
def save_info_dict(info_dict_tmp):
  for k in info_dict_tmp.keys():
    if 'numpy.ndarray' in str(type(info_dict_tmp[k])):
      info_dict_tmp[k] = info_dict_tmp[k].tolist()
  with open(info_dict_tmp['model_save_dir'] + 'info_dict' + '.json', 'w') as f:
          json.dump(info_dict_tmp, f)
def foo_save_and_plot(training_info, save_and_plot_history_1, save_loc):
    text_str = transfer_learning.make_text_for_fig(training_info)
    training_info = transfer_learning.pack_training_info(training_info, save_and_plot_history_1)
    # save_num = foo_get_next_save_num(save_loc)

    # make figure and save
    transfer_learning.plot_train_hist(training_info, [1], [.9, 1], text_str)
    plt.savefig(save_loc + 'mod_test_fig_ACC' + '.png', bbox_inches="tight")
    transfer_learning.plot_train_hist(training_info, [0], [0, 0.25], text_str)
    plt.savefig(save_loc + 'mod_test_fig_LOSS' + '.png', bbox_inches="tight")

    # save training info
    with open(save_loc + 'model_eval_each_epoch' + '.json', 'w') as f:
        json.dump(training_info, f)


# In[17]:


def foo_run_all():
  #copy and unzip for colab
  global all_models_directory, test_data_dir, info_dict, transfer_learning

  local_dict = make_initial_folder(all_models_directory,
                                  unique_h5_train_val_dir)  # make data single frame directory and get that directory
  info_dict = info_dict_wrapper(info_dict, local_dict)  # wrap output into a dict

  BASE_H5 = info_dict['all_models_directory'] + os.sep + info_dict['unique_h5_train_val_dir']  # directory for all the data for a certain type og images(lag or regular etc)

  local_dict = get_automated_model_info(BASE_H5, image_source_h5_directory_ending, test_data_dir)  # basic data like directories and train and val set (automated form the directory)
  info_dict = info_dict_wrapper(info_dict, local_dict)  # wrap output into a dict

  # """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
  local_dict = copy_over_new_labels(label_key_name, info_dict['image_h5_list'] + [info_dict['h5_test']],
                                    info_dict['label_h5_list'] + [info_dict[ 'h5_test_labels']])  # copy specific labels to the H5 of interest (this is all after the conversion from single frame)
  info_dict = info_dict_wrapper(info_dict, local_dict)  # wrap output into a dict

  # """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$      make model     $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
  # get all the labels
  train_gen = image_tools.ImageBatchGenerator(batch_size, [info_dict['h5_train']])
  val_gen = image_tools.ImageBatchGenerator(batch_size, [info_dict['h5_val']])
  test_gen = image_tools.ImageBatchGenerator(batch_size, [info_dict['h5_test']])

  train_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_train']])
  val_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_val']])
  labels = np.concatenate((train_labels, val_labels))

  model, class_weights, info_dict = build_model(info_dict, labels, model_name_str,
                                                base_learning_rate=base_learning_rate,
                                                dropout_val=dropout_val)

#   info_dict['BASE_H5'] = change_to_gdrive_dir(info_dict['BASE_H5'])
  model_save_dir = make_model_save_directory(info_dict)  # make a unique folder using standard folder struct ending in a unique date/time folder
  # ###change directory to gdrive in case it crashes
  # dir_2_change = ['model_save_dir_checkpoints', 'model_save_dir']
  # for k in dir_2_change:
  #   info_dict[k] = change_to_gdrive_dir(info_dict[k])

  model, class_weights, info_dict = build_model(info_dict,
                                                labels,
                                                model_name_str,
                                                base_learning_rate=base_learning_rate,
                                                dropout_val=dropout_val)

  callbacks = basic_callbacks(info_dict['model_save_dir_checkpoints'], monitor=monitor, patience=patience,
                              save_best_only=save_best_only, save_weights_only=True,
                              save_freq=save_freq, period=period)

  plot_callback = transfer_learning.save_and_plot_history(test_gen, key_name='labels', plot_metric_inds=[0])
  callbacks.append(plot_callback)

  ###FITTTTTTT
  history = model.fit(train_gen,
                      epochs=epochs,
                      validation_data=val_gen,
                      callbacks=callbacks,
                      class_weight=class_weights)
  #save finals checkpoint after model finishes
  model.save_weights(info_dict['model_save_dir_checkpoints']+os.sep+'final_epoch_cp.hdf5')

  training_info = pack_training_info(plot_callback)

  xx = '/content/colab_data2/model_testing/all_data/all_models/small_h5s/InceptionV3/3lag/on-off_set_and_one_after/'
  utils.get_files(xx, '*_cp.hdf5')

  transfer_learning.foo_save_and_plot

  foo_save_and_plot(training_info, plot_callback, info_dict['model_save_dir'])

  save_info_dict(info_dict)
  ### dont need to copy we changed the directory above
  # x1 = info_dict['model_save_dir']
  # x2 = change_to_gdrive_dir(x1)
  # shutil.copytree(x1, x2)


# In[18]:


## start

label_key_name_list = label_naming_shorthand_dict()  # get a list of label key names... they are really long to be specific
utils.print_list_with_inds(label_key_name_list)  # print them, then below use their index to choose them

all_models_directory = bd2+"model_testing/all_data/all_models/"  # DETERMINES location for all model you will run
test_data_dir = bd2+"model_testing/all_data/test_data/10_percent_holy_set/"  # ^^^^^^^DETERMINES^^^^^^^ location for test data (folder determined by the "image_source_h5_directory_ending" variable)
unique_h5_train_val_dir = 'regular_80_border'  # ^^^^^^^DETERMINES^^^^^^^ the name of the folder where each type of data is stored
image_source_h5_directory_ending = "/3lag/"  # ^^^^^^^DETERMINES^^^^^^^ THE IMAGE SOURCE
label_key_name = list(label_key_name_list.keys())[2]  # ^^^^^^^DETERMINES^^^^^^^ THE LABEL SOURCE choose the ind based on the print out

re_copy_and_unzip = True

model_name_str = 'ResNet50V2'  # ^^^^^^^DETERMINES^^^^^^^ model base
base_learning_rate = 10**-6  # DETERMINES rate of change for each epoch step
dropout_val = 0.5  # DETERMINES percentage of dropout for training data
patience = 15  # DETERMINES early stopping
save_freq = "epoch"  # leave this as epoch
period = 2 # DETERMINES how often it saves the checkpoints
epochs = 5000  # DETERMINES how many epochs the model trains for if early stopping is never triggered
batch_size = 200  # DETERMINES number of images per batch
save_best_only = True
monitor = 'val_loss'

info_dict = dict()


##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################

from twilio.rest import Client

def send_text(mess):
  account_sid = "AC403675d69d88a93207f1cb80e9187c5e"
  auth_token = '111876a8b222ce0eff99b2ddaa47f2fb'
  client = Client(account_sid, auth_token)
  message = client.messages .create(
                      body =  mess, #Message you send
                      from_ = "+19199754333",#Provided phone number
                      to =    "+15023109622")#Your phone number
  message.sid


label_ind = 0
unique_h5_train_val_dir = 'regular_80_border'  # ^^^^^^^DETERMINES^^^^^^^ the name of the folder where each type of data is stored
image_source_h5_directory_ending = "/3lag/"  # ^^^^^^^DETERMINES^^^^^^^ THE IMAGE SOURCE
# 0 [0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)
# 1 [0, 1, 2, 3]- (no touch, touch, onset, offset
# 2 [0, 1, 2]- (no event,from twilio.rest import Client
#
# def send_text(mess):
#   account_sid = "AC403675d69d88a93207f1cb80e9187c5e"
#   auth_token = '111876a8b222ce0eff99b2ddaa47f2fb'
#   client = Client(account_sid, auth_token)
#   message = client.messages .create(
#                       body =  mess, #Message you send
#                       from_ = "+19199754333",#Provided phone number
#                       to =    "+15023109622")#Your phone number
#   message.sid
#
#
# label_ind = 0
# unique_h5_train_val_dir = 'regular_80_border'  # ^^^^^^^DETERMINES^^^^^^^ the name of the folder where each type of data is stored
# image_source_h5_directory_ending = "/3lag/"  # ^^^^^^^DETERMINES^^^^^^^ THE IMAGE SOURCE
# # 0 [0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)
# # 1 [0, 1, 2, 3]- (no touch, touch, onset, offset
# # 2 [0, 1, 2]- (no event, onset, offset)
# # 3 [0, 1]- (no touch, touch)
# # 4 [0, 1]- (not offset, offset)
# # 5 [0, 1]- (not onset, onset)
# label_key_name = list(label_key_name_list.keys())[label_ind]  # ^^^^^^^DETERMINES^^^^^^^ THE LABEL SOURCE choose the ind based on the print out
#
#
# batch_size = 2**8
#
# info_dict = dict()
# model_name_str = 'MobileNetV3Small'  # ^^^^^^^DETERMINES^^^^^^^ model base
# send_text('starting model --> ' + model_name_str)
# foo_run_all()
#
#
# info_dict = dict()
# model_name_str = 'InceptionV3'  # ^^^^^^^DETERMINES^^^^^^^ model base
# send_text('starting model --> ' + model_name_str)
# foo_run_all()
#
# info_dict = dict()
# model_name_str = 'MobileNetV3Large'  # ^^^^^^^DETERMINES^^^^^^^ model base
# send_text('starting model --> ' + model_name_str)
# foo_run_all()
#
# info_dict = dict()
# model_name_str = 'ResNet50V2'  # ^^^^^^^DETERMINES^^^^^^^ model base
# send_text('starting model --> ' + model_name_str)
# foo_run_all()
#
# # did not finish
# patience = 6  # DETERMINES early stopping
# batch_size = 2**6  # DETERMINES number of images per batch
# info_dict = dict()
# model_name_str = 'EfficientNetB7'  # ^^^^^^^DETERMINES^^^^^^^ model base
# send_text('starting model --> ' + model_name_str)
# foo_run_all()
#
#
#
# send_text('FINISHED FINISHED FINISHED FINISHED FINISHED FINISHED FINISHED FINISHED FINISHED ')
# send_text('cuda xxx') onset, offset)
# 3 [0, 1]- (no touch, touch)
# 4 [0, 1]- (not offset, offset)
# 5 [0, 1]- (not onset, onset)
label_key_name = list(label_key_name_list.keys())[label_ind]  # ^^^^^^^DETERMINES^^^^^^^ THE LABEL SOURCE choose the ind based on the print out


batch_size = 2**8

info_dict = dict()
model_name_str = 'MobileNetV3Small'  # ^^^^^^^DETERMINES^^^^^^^ model base
send_text('starting model --> ' + model_name_str)
foo_run_all()


info_dict = dict()
model_name_str = 'InceptionV3'  # ^^^^^^^DETERMINES^^^^^^^ model base
send_text('starting model --> ' + model_name_str)
foo_run_all()

info_dict = dict()
model_name_str = 'MobileNetV3Large'  # ^^^^^^^DETERMINES^^^^^^^ model base
send_text('starting model --> ' + model_name_str)
foo_run_all()

info_dict = dict()
model_name_str = 'ResNet50V2'  # ^^^^^^^DETERMINES^^^^^^^ model base
send_text('starting model --> ' + model_name_str)
foo_run_all()

# did not finish
patience = 6  # DETERMINES early stopping
batch_size = 2**6  # DETERMINES number of images per batch
info_dict = dict()
model_name_str = 'EfficientNetB7'  # ^^^^^^^DETERMINES^^^^^^^ model base
send_text('starting model --> ' + model_name_str)
foo_run_all()



send_text('FINISHED FINISHED FINISHED FINISHED FINISHED FINISHED FINISHED FINISHED FINISHED ')
send_text('cuda xxx')





from whacc import utils
from whacc import image_tools

"""
from google.colab import drive 
drive.mount('/content/gdrive')
"""

# This is all that is needed to make a master H5 of near touches only, the iamges do not have color chanels to save space.
all_h5s = utils.get_h5s('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/finished_contacts/')
all_h5s_imgs = utils.get_h5s('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/H5_data/')
h_cont = utils._get_human_contacts_(all_h5s)
h5c = image_tools.h5_iterative_creator('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/test_____.h5',
                                       overwrite_if_file_exists = True,
                                       color_channel = False)
utils.create_master_dataset(h5c, all_h5s_imgs, h_cont, borders = 80, max_pack_val = 100)
